@@ -7,11 +7,14 @@ handle_error_code <- function(status_code) {
   explanation <- switch(as.character(status_code),
                         "401" = "Authentication invalid or key incorrect or not part of the organization.",
                         "429" = "Rate limit reached or quota exceeded.",
-                        "500" = "Server error during request processing.",
-                        "503" = "Engine overloaded, try again later.",
-                        "404" = "check your url address",
-                        "400" = "This request error",
-                        default = "Unknown error."
+                        "403" = "Access forbidden. You may not have the necessary permissions.",
+                        "500" = "Server error during request processing. You may want to try again later.",
+                        "503" = "Service unavailable or engine overloaded. Try again later.",
+                        "404" = "Resource not found. Please check your URL address.",
+                        "400" = "Bad request. Please check your request format.",
+                        "502" = "Bad Gateway. The server received an invalid response from the upstream server.",
+                        "503" = "Service unavailable. Please try again later.",
+                        default = paste("Unknown error. Status code:", status_code)
   )
 
   message(explanation)
