@@ -87,14 +87,16 @@ message(body)
 openai_chat <- function(
     messages = list(list(role = "user", content = "Please repeat 'Setup Successful'. DON'T say anything else.")),
     model = Sys.getenv("model"),
+    max_retries = 6000, 
+    initial_wait_time = 10,  
+    max_wait_time = 6000, 
     ...
 ){
-  
+
   if (is.null(Sys.getenv("key"))) {
     stop("API key is not set.")
   }
 
-
   args <- list(
     model = model,
     messages = messages,
@@ -104,62 +106,72 @@ openai_chat <- function(
   content_list <- NULL  
   response_text <- NULL  
   response_content <- NULL
+  wait_time <- initial_wait_time
   
-  
-  tryCatch({
-    
-    if (Sys.getenv("LOG_FILE") != "") {
-      log_file_path <- Sys.getenv("LOG_FILE")
-      log_file_connection <- file(log_file_path, open = "a")
-      sink(log_file_connection, append = TRUE, type = "message")
-      res <- POST(
-        url = Sys.getenv("url"),
-        body = toJSON(args),
-        add_headers(
-          'Content-Type' = 'application/json',
-          'Authorization' = paste('Bearer', Sys.getenv("key")),
-          'X-use-cache' = "false"),
-        verbose(data_out = TRUE, data_in = FALSE, info = FALSE)
-      )
-      sink(type = "message")
-    } else {
-      res <- POST(
-        url = Sys.getenv("url"),
-        body = toJSON(args),
-        add_headers(
-          'Content-Type' = 'application/json',
-          'Authorization' = paste('Bearer', Sys.getenv("key")),
-          'X-use-cache' = "false"
+  for (attempt in 1:max_retries) {
+    tryCatch({
+      
+      if (Sys.getenv("LOG_FILE") != "") {
+        log_file_path <- Sys.getenv("LOG_FILE")
+        log_file_connection <- file(log_file_path, open = "a")
+        sink(log_file_connection, append = TRUE, type = "message")
+        res <- POST(
+          url = Sys.getenv("url"),
+          body = toJSON(args),
+          add_headers(
+            'Content-Type' = 'application/json',
+            'Authorization' = paste('Bearer', Sys.getenv("key")),
+            'X-use-cache' = "false"
+          ),
+          verbose(data_out = TRUE, data_in = FALSE, info = FALSE)
         )
-      )}
-    # message("openai_chat_Model: ", model)
-    # message("openai_chat_Url: ", Sys.getenv("url"))
-    # message("openai_chat_Args: ", args)
-    # message("openai_chat_Key: ", Sys.getenv("key"))
-    # message("openai_chat_Message: ", toJSON(modifyList(args, list(...))))
+        sink(type = "message")
+      } else {
+        res <- POST(
+          url = Sys.getenv("url"),
+          body = toJSON(args),
+          add_headers(
+            'Content-Type' = 'application/json',
+            'Authorization' = paste('Bearer', Sys.getenv("key")),
+            'X-use-cache' = "false"
+          )
+        )
+      }
+      
+      if(res$status_code == 200 || res$status_code == 201){
+        response_text <- content(res, 'text', encoding = "UTF-8")
+        res_json <- fromJSON(response_text)
+        choices <- res_json$choices
+        content_list <- sapply(choices, function(x) x$message$content)
+        break  
+      }
+
+      if(res$status_code == 429) {
+        message(paste("Too many requests. Retrying in", wait_time, "seconds..."))
+        Sys.sleep(wait_time)
+        wait_time <- min(wait_time + wait_time, max_wait_time)  
+      } else {
+        response_content <- paste("error with code:", res$status_code)
+        message(response_content)
+        message(content(res, 'text', encoding = "UTF-8"))
+        stop(paste("Request failed with status code", res$status_code))
+      }
+      
+    }, warning = function(war) {
+      message(paste("Caught warning:", war$message))
+    }, error = function(err) {
+      if (!is.null(response_content)) {
+        message(response_content)
+      }
+      stop(err)
+    })
     
-    if(res$status_code != 200 && res$status_code != 201){
-      response_content <- paste("error with code:", res$status_code)
-      message(response_content)
-      message(content(res, 'text', encoding = "UTF-8"))
-      stop(paste("Request failed with status code", res$status_code))
+    if (attempt == max_retries) {
+      stop("Maximum retries exceeded. Unable to complete the request.")
     }
-    
-    response_text <- content(res, 'text', encoding = "UTF-8")
-    res_json <- fromJSON(response_text)
-    choices <- res_json$choices
-    content_list <- sapply(choices, function(x) x$message$content)
-  }, warning = function(war) {
-    message(paste("Caught warning:", war$message))
-  }, error = function(err) {
-    if (!is.null(response_content)) {
-      message(response_content)
-    }
-    stop(err)
-  })
+  }
   
   result_list <- list(content_list = content_list, raw_response = response_text)
-  
   return(result_list)
 }
 
@@ -177,8 +189,9 @@ openai_chat <- function(
 #' @return A list of character strings containing the claude's responses.
 #' @noRd
 claude_chat <- function(
-  messages = list(),
+  messages = list(list(role = "user", content = "Please repeat 'Setup Successful'. DON'T say anything else.")),
   model = "claude-3-sonnet-20240229",
+  max_tokens = 1024,
   version = "2023-06-01",
   ...
 ){
@@ -188,17 +201,17 @@ claude_chat <- function(
   }
 
 
-
-
   headers <- c(
     'x-api-key' = Sys.getenv("key"),
     'anthropic-version' = version,
-    'Content-Type' = 'application/json'
+    'content-type' = 'application/json'
   )
 
   args <- list(
+    model = model,
     messages = messages,
-    model = model
+    max_tokens = max_tokens,
+    ...
   )
 
 
@@ -206,11 +219,12 @@ claude_chat <- function(
 
     res <- POST(
       url = Sys.getenv("url"),
-      body = modifyList(args, list(...)),
       add_headers(headers),
+      body = toJSON(args),
       encode = "json",
       config = config(ssl_verifypeer = 0L, timeout = 300)
     )
+    
     if(res$status_code != 200 && res$status_code != 201){
       response_content = paste("error with code:",res$status_code)
       message(response_content)
@@ -256,7 +270,7 @@ claude_chat <- function(
 #' @return A list of character strings containing the gemini's responses.
 #' @noRd
 gemini_chat <- function(
-  messages = list(list(parts = list(list(text = "Please repeat 'Setup successful'. DON'T say anything else at the beginning of your reponse.")))),
+  messages = list(list(parts = list(text = "Please repeat 'Setup successful'. DON'T say anything else at the beginning of your reponse."))),
   model = Sys.getenv("model"),
   ...
 ){
@@ -264,15 +278,41 @@ gemini_chat <- function(
   if (is.null(Sys.getenv("key"))) {
     stop("API key is not set.")
   }
+  gptConfig <- tryCatch(get("gptConfig", envir = .GlobalEnv), error = function(e) NULL)
+  if (exists("gptConfig")) {
+    if (!is.null(gptConfig$systemPrompt) && gptConfig$systemPrompt != "") {
+      system_instruction <- list(parts = list(text = gptConfig$systemPrompt))
+    } else {
+      system_instruction <- ""
+    }
+  } else {
+    system_instruction <- ""
+  }
 
   generationConfig = list(...)
 
-  body <- list(
-    contents = messages,
-    model=model,
-    generationConfig = generationConfig
-  )
-
+  if (system_instruction == "") {
+    body <- list(
+      contents = messages,
+      model = model
+    )
+    
+    if (!is.null(generationConfig)) {
+      body$generationConfig <- generationConfig
+    }
+    
+  } else {
+    body <- list(
+      system_instruction = system_instruction,
+      contents = messages,
+      model = model
+    )
+    
+    if (!is.null(generationConfig)) {
+      body$generationConfig <- generationConfig
+    }
+  }
+# message(body)
 
   current_url <- paste0("https://generativelanguage.googleapis.com/v1beta/models/",model,":generateContent?key=",Sys.getenv("key"))
 
@@ -299,9 +339,6 @@ gemini_chat <- function(
         #verbose(data_out = TRUE, data_in = FALSE, info = FALSE)
       )
       }
-    
-
-    
 
 
     if(res$status_code != 200 && res$status_code != 201){
@@ -330,9 +367,6 @@ gemini_chat <- function(
       content_text <- parts[[1]]$text
     }
 
-
-
-
   }, warning = function(war) {
     message(paste("Caught warning:", war$message))
 
@@ -503,10 +537,14 @@ openai_completion <- function(
 
 wenxin_chat <- function(messages = list(list(role = "user", content = "Please repeat 'Setup successful'. DON'T say anything else.")), ...) {
   model = Sys.getenv("model")
+  if (tolower(model) == "yi-34b-chat") {
+    model <- "yi_34b_chat"
+  }
   api_key = Sys.getenv("key")
   secret_key = Sys.getenv("secret_key")
   url = Sys.getenv("url")
   
+  
   get_access_token <- function(api_key, secret_key) {
     url_access_token <- paste0("https://aip.baidubce.com/oauth/2.0/token?grant_type=client_credentials&client_id=", api_key, "&client_secret=", secret_key)
     res <- GET(url_access_token)
@@ -523,13 +561,13 @@ wenxin_chat <- function(messages = list(list(role = "user", content = "Please re
   
   headers <- c('Content-Type' = 'application/json')
   
-  message("wenxin_chat: messages: ", messages)
+  #message("wenxin_chat: messages: ", messages)
   args <- list(
     messages = messages,
     ...
   )
   
-  message("wenxin_chat: args: ", args)
+  #message("wenxin_chat: args: ", args)
 
 
   tryCatch({
@@ -541,7 +579,7 @@ wenxin_chat <- function(messages = list(list(role = "user", content = "Please re
       encode = "json",
       config = config(ssl_verifypeer = 0L, timeout = 300)
     )
-    message(res)
+    #message(res)
     
     if (res$status_code != 200 && res$status_code != 201) {
       response_content = paste("Error with code:", res$status_code)
@@ -551,7 +589,10 @@ wenxin_chat <- function(messages = list(list(role = "user", content = "Please re
     }
     
     response_text <- content(res, 'text', encoding = "UTF-8")
+    
+    
     res_json <- fromJSON(response_text)
+    
     # message(res_json)
     result <- list(content_list = res_json$result, raw_response = response_text)
   }, warning = function(war) {
